﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BlacksmithWorkshopDatabaseImplement.Models
{
	public class MessageInfo : IMessageInfoModel
	{
		[Key]
		public string MessageId { get; private set; } = string.Empty;

		public int? ClientId { get; private set; }

		[Required]
		public string SenderName { get; private set; } = string.Empty;

		[Required]
		public DateTime DateDelivery { get; private set; } = DateTime.Now;

		[Required]
		public string Subject { get; private set; } = string.Empty;

		[Required]
		public string Body { get; private set; } = string.Empty;

		public virtual Client? Client { get; set; }

		public static MessageInfo? Create(MessageInfoBindingModel model)
		{
			if (model == null)
			{
				return null;
			}

			return new MessageInfo()
			{
				MessageId = model.MessageId,
				ClientId = model.ClientId,
				SenderName = model.SenderName,
				Body = model.Body,
				DateDelivery = model.DateDelivery,
				Subject = model.Subject
			};
		}

		public void Update(MessageInfoBindingModel model)
		{
			if (model == null)
			{
				return;
			}

			MessageId = model.MessageId;
			ClientId = model.ClientId;
			SenderName = model.SenderName;
			Body = model.Body;
			DateDelivery = model.DateDelivery;
			Subject = model.Subject;
		}

		public MessageInfoViewModel GetViewModel => new()
		{
			MessageId = MessageId,
			ClientId = ClientId,
			SenderName = SenderName,
			Body = Body,
			DateDelivery = DateDelivery,
			Subject = Subject
		};
	}
}
